package com.book.server.controller;

import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.common.entity.ResultCode;
import com.book.server.dao.entity.Book;
import com.book.server.dao.entity.SearchKeyword;
import com.book.server.model.VO.BlockRes;
import com.book.server.model.VO.BookRes;
import com.book.server.model.VO.Chapter;
import com.book.server.model.VO.QueryVO;
import com.book.server.service.BookService;
import com.mysql.cj.QueryBindings;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * 书籍
 */
@RestController
@RequestMapping("/api/book")
@Slf4j
public class BookController extends BaseController {
    @Autowired
    private BookService bookService;


    /**
     * 书库查询
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/query")
    public Result<List<BookRes>> query(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.query(queryVO));
    }

    /**
     * 书城分页推荐
     *
     * @param queryVO
     * @return
     */
    @PostMapping("/recommend")
    public Result<List<Book>> recommendByRead(@RequestBody QueryVO queryVO) {
        return Result.success(bookService.recommendByRead(queryVO));
    }

    /**
     * 热门推荐4个
     * @param queryVO
     * @return
     */
    @PostMapping("/topRecommend")
    public Result<List<Book>> topRecommend(@RequestBody QueryVO queryVO) {
        queryVO.setPage(0);
        queryVO.setSize(10);
        queryVO.setPageId(queryVO.getSex()); // 为了复用recommendByRead的代码
        List<Book> books = bookService.recommendByRead(queryVO);
        Collections.shuffle(books);
        return Result.success(books.subList(0, 4));
    }

    /**
     * 书城block
     *
     * @param pageId
     * @return
     */
    @GetMapping("/getBlockByPageId")
    @Cacheable(value = "block", key = "#pageId")
    public Result<List<BlockRes>> getBlockByPageId(Integer pageId) {
        List<BlockRes> list = bookService.getBlockByPageId(pageId);
        return Result.byObject(list);
    }

    /**
     * 更新block缓存
     *
     * @return
     */
    @GetMapping("/resetBlockCache")
    @CacheEvict(value = "block", allEntries = true)
    public Result resetBlockCache() {
        return Result.byObject("OK");
    }


    /**
     * 导入书库
     * 注意：此代码仅限拉取书籍使用，只能运行一次。
     *
     * @return
     */

    @GetMapping("/insertBooks")
    public Result insertBooks() {
        boolean open = false;
        if (open) {
            bookService.insertBooks();
        }
        return Result.byObject("OK");
    }

    /**
     * 获取章节列表
     * @param book
     * @return
     */
    @PostMapping("/chapterList")
    public Result<List<Chapter>> getChaptersByPageAndRow(@RequestBody Map<String,String> book) {

      List<Chapter>  chapterList =  bookService.getChaptersByBookId(book.get("bookId"));
      if ( CollectionUtils.isEmpty(chapterList)){
          return Result.failure(ResultCode.FAIL.getCode(),"哎呀~没有数据啦~");
      }else {
          return Result.byObject(chapterList);
      }
    }

    /**
     * 获取热门搜索
     * @param sex
     * @return
     */
    @GetMapping("/getSearchKeyword")
    public Result<List<SearchKeyword>> getSearchKeywordBySex(String sex) {
        return Result.byObject(bookService.getSearchKeywordBySex(sex));
    }

}
